//main.cpp
#include <iostream>
#include <functional>
#include <algorithm>
#include <map>
#include <stdio.h>
#include "StriLess.h"

class IFunctionImp
{
public:
     virtual void Process() = 0;
};

template<class T>
class FunctionImp : public IFunctionImp
{
public:
    typedef void (T::*Callback)();

    FunctionImp(T * obj, Callback callback) : m_pOobj(obj), m_callback(callback)
    {
    }

    void Process()
    {
         (m_pOobj->*m_callback)();
    }

private:
     T * m_pOobj;
     Callback m_callback;
};

class Lion
{
public:	
	void Run() 
	{
		std::cout << "Lion run" << std::endl; 
	}

	void Attack() 
	{
		std::cout << "Lion attack" << std::endl; 
	}
};

class Tiger
{
public:
	void Idle() 
	{
		std::cout << "Tiger idle" << std::endl; 
	}
};



class Command;

typedef std::pair<const char*, IFunctionImp*>            MakeCmd;
typedef std::map<const char*, IFunctionImp*, StriLess >  CmdMap;


class Command
{
	CmdMap cmdTable;
public:
	Command(){} ;
	virtual ~Command(){ Release(); }
	void	Release();

	void Add( const char* strCmd, IFunctionImp* imp);
	void Process( char* str );
};


void Command::Release()
{
	CmdMap::iterator endItor = cmdTable.end();

	for( CmdMap::iterator itor = cmdTable.begin(); itor != endItor; ++itor )
	{
		delete itor->second;
	}

	cmdTable.clear();
}

void Command::Add( const char* strCmd, IFunctionImp* imp)
{
	cmdTable.insert( MakeCmd( strCmd, imp ) );
}

void Command::Process( char* str )
{
	CmdMap::iterator iter = cmdTable.find( str );
	if( iter != cmdTable.end() )
	{
		(iter->second)->Process(); 
	}
}


#define MakeFunc( a, b, c )   new FunctionImp<b>( &a, &b::c)

void main()
{
	Command command;

	Lion lion;
	command.Add( "Run", new FunctionImp<Lion>(&lion, &Lion::Run) );
	Tiger tiger;
	command.Add( "Idle", new FunctionImp<Tiger>(&tiger, &Tiger::Idle) );

	command.Add( "Attack", MakeFunc( lion, Lion, Attack ) );

	command.Process( "Run" );
	command.Process( "Idle" );
	command.Process( "Attack" );
}